﻿using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Linq;
using System.Web;
using System.Web.Configuration;
using System.Web.Script.Serialization;
using System.Web.Script.Services;
using System.Web.Services;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace GanttSample
{
    public partial class GanttSample : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            var DataSource = this.GetDataSource();
            this.Gantt.DataSource = DataSource;
            this.Gantt.Resources = dropdown();
            this.Gantt.DataBind();
        }

        public class myDropDown {
            public string ddvalue { get; set; }
            public string ddtext {get; set;}
        }
        [WebMethod]

        [ScriptMethod(ResponseFormat = ResponseFormat.Json)]

        public static IEnumerable<object> dropdown()
        {
            // instantiate a serializer
            JavaScriptSerializer TheSerializer = new JavaScriptSerializer();
            List<myDropDown> resourceDetails = new List<myDropDown>();
            resourceDetails.Add(new myDropDown() { ddvalue = "1", ddtext = "Project Manager" });
            resourceDetails.Add(new myDropDown() { ddvalue = "2", ddtext = "Software Analyst" });
            resourceDetails.Add(new myDropDown() { ddvalue = "3", ddtext = "Developer" });
            resourceDetails.Add(new myDropDown() { ddvalue = "4", ddtext = "Testing Engineer" });
            var resourceList = resourceDetails.ToList();
            var theJson = TheSerializer.Serialize(resourceDetails);
            return resourceList;
        }

        private List<BusinessObject> GetDataSource()
        {
            List<BusinessObject> list = new List<BusinessObject>();

            SqlConnection con = new SqlConnection(WebConfigurationManager.ConnectionStrings["DefaultConnection"].ConnectionString);//connectionString
            con.Open();
            
            using (con)
            {
                using (var command = con.CreateCommand())
                {
                    command.CommandText = "SELECT * FROM GanttData";

                    using (var reader = command.ExecuteReader())
                    {
                        var indexOfCol1 = reader.GetOrdinal("TaskId");
                        var indexOfCol2 = reader.GetOrdinal("TaskName");
                        var indexOfCol3 = reader.GetOrdinal("StartDate");
                        var indexOfCol4 = reader.GetOrdinal("EndDate");
                        var indexOfCol5 = reader.GetOrdinal("Progress");
                        var indexOfCol6 = reader.GetOrdinal("Duration");
                        var indexOfCol7 = reader.GetOrdinal("ParentId");
                        var indexOfCol8 = reader.GetOrdinal("Resources");

                        while (reader.Read())
                        {
                            BusinessObject obj = new BusinessObject();

                            obj.Id = reader.GetValue(indexOfCol1).ToString();
                            obj.ParentId = reader.GetValue(indexOfCol7).ToString();
                            obj.Name = reader.GetValue(indexOfCol2).ToString();
                            obj.PercentDone = reader.GetValue(indexOfCol5).ToString();
                            obj.StartDate = reader.GetValue(indexOfCol3).ToString();
                            obj.EndDate = reader.GetValue(indexOfCol4).ToString();
                            obj.ResourceID = this.GetResourceValues(reader.GetValue(indexOfCol8).ToString());
                            list.Add(obj);

                        }
                        reader.Close();
                    }
                }
                con.Close();

            }
            return list;
        }

        public List<string> GetResourceValues(string resourceArr)
        {
            List<string> Arr = new List<string>();
            string[] resourceId = resourceArr.Split(',');
            IEnumerable<object>dropdownData = dropdown();
            List<myDropDown> resultData=dropdownData as List<myDropDown>;
            foreach (var id in resourceId)
            {
                foreach (var data in resultData)
                {
                    if (id == data.ddvalue)
                    {
                        Arr.Add(data.ddvalue);
                    }
                }
            }
           

            return Arr;
        }

        public void Update(BusinessObject Task)
        {
            //Update Edited TaskDetails
            //Using UPDATE SQL Query to Save the Modified taskData into the Table

            SqlConnection con = new SqlConnection(WebConfigurationManager.ConnectionStrings["DefaultConnection"].ConnectionString);//connectionString
            con.Open();

            string IDNumber = Task.Id;

            string cmdString = "UPDATE GanttData SET TaskId=@TaskId, TaskName=@TaskName,StartDate=@StartDate,EndDate=@EndDate,Progress=@Progress,ParentId=@ParentId,Resources=@Resource WHERE TaskId = '" + IDNumber + "'";


            using (SqlCommand sqlCommand = new SqlCommand(cmdString, con))
            {
                sqlCommand.Parameters.AddWithValue("@TaskName", Task.Name);

                sqlCommand.Parameters.AddWithValue("@StartDate",this.GetDate(Task.StartDate));
                sqlCommand.Parameters.AddWithValue("@TaskId", Task.Id);
                sqlCommand.Parameters.AddWithValue("@EndDate",this.GetDate(Task.EndDate));
                sqlCommand.Parameters.AddWithValue("@Progress", Task.PercentDone);
                var ResourceList=this.GetResourceValues(string.Join(",", Task.ResourceID.ToArray()));
                sqlCommand.Parameters.AddWithValue("@Resource",string.Join(",",ResourceList.ToArray()));
                if (Task.ParentId == null)
                {
                    sqlCommand.Parameters.AddWithValue("@ParentId", Task.ParentId).Value = "0";
                }
                else
                {
                    sqlCommand.Parameters.AddWithValue("@ParentId", Task.ParentId);
                }

                sqlCommand.ExecuteNonQuery();
            }

            con.Close();
        }

        public string GetDate(string date)
        {
            DateTime convertedDate = Convert.ToDateTime(date);

            return (convertedDate.Month + "/" + convertedDate.Day + "/" + convertedDate.Year);
        }

        public void Add(BusinessObject Task)
        {

            //Here Task is the added New Task
            //Add Task 
            //Using INSERT Query to add the New Record to SQL Table

          

        }
        public void Delete(BusinessObject Task)
        {

            //Delete Task 
            //Using Delete Query to delete Record from SQL Table


        }


        [WebMethod]
        public static void AddIt(BusinessObject record)
        {
            GanttSample sample = new GanttSample();
            sample.Add(record);

        }
        [WebMethod]
        public static void UpdateIt(BusinessObject record)
        {
            GanttSample sample1 = new GanttSample();
            sample1.Update(record);

        }
        [WebMethod]
        public static void DeleteIt(BusinessObject record)
        {
            GanttSample sample2 = new GanttSample();
            sample2.Delete(record);

        }



        #region BusinessObject Creation
        public class BusinessObject
        {
            public string StartDate { get; set; }
            public string Id { get; set; }
            public string Name { get; set; }
            public string EndDate { get; set; }
            public string PercentDone { get; set; }
            public List<string> ResourceID { get; set; }
            //public List<BusinessObject> Children { get; set; }
            public string ParentId { get; set; }
        }
        #endregion
    }
}